import webbrowser

class Websites:
    @staticmethod
    def open_website(url=""):
        """
        Opens a website in the default web browser.

        Args:
        - url: The URL of the website to open.

        Returns:
        - 'opened' if successful.
        - Error message if unsuccessful.
        """
        try:
            webbrowser.open(url)
            return "opened"
        except Exception as e:
            print("An error occurred:", e)
            return "An error occurred:", e


from cryptography.fernet import Fernet
from jose import JWTError, jwt
from passlib.context import CryptContext
import secrets
import base64
from datetime import datetime, timedelta
import hashlib
import re
import os
import string
import random
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa, padding
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from typing import Union, List, Tuple
import ipaddress
import urllib.parse

class Security:
    """
    Provides comprehensive security features including encryption, hashing, JWT handling, password management,
    key management, certificate operations, and various security utilities.
    """

    def __init__(self, secret_key: str = None):
        """Initialize the Security class with optional secret key."""
        self.secret_key = secret_key or secrets.token_urlsafe(32)
        self.pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
        self.fernet = Fernet(base64.b64encode(self.secret_key.encode()[:32].ljust(32, b'0')))
        self._initialize_rsa_keys()

    def _initialize_rsa_keys(self):
        """Initialize RSA key pair for asymmetric encryption."""
        self.private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048
        )
        self.public_key = self.private_key.public_key()

    def hash_password(self, password: str) -> str:
        """Hash a password using bcrypt."""
        return self.pwd_context.hash(password)

    def verify_password(self, plain_password: str, hashed_password: str) -> bool:
        """Verify a password against its hash."""
        return self.pwd_context.verify(plain_password, hashed_password)

    def create_jwt_token(self, data: dict, expires_delta: timedelta = None) -> str:
        """Create a JWT token with optional expiration."""
        to_encode = data.copy()
        expire = datetime.utcnow() + (expires_delta or timedelta(minutes=15))
        to_encode.update({"exp": expire})
        return jwt.encode(to_encode, self.secret_key, algorithm="HS256")

    def verify_jwt_token(self, token: str) -> dict:
        """Verify and decode a JWT token."""
        try:
            return jwt.decode(token, self.secret_key, algorithms=["HS256"])
        except JWTError as e:
            raise e

    def encrypt_data(self, data: str) -> str:
        """Encrypt data using Fernet symmetric encryption."""
        encrypted_data = self.fernet.encrypt(data.encode())
        return base64.b64encode(encrypted_data).decode()

    def decrypt_data(self, encrypted_data: str) -> str:
        """Decrypt Fernet-encrypted data."""
        try:
            decrypted_data = self.fernet.decrypt(base64.b64decode(encrypted_data))
            return decrypted_data.decode()
        except Exception as e:
            raise ValueError(f"Decryption failed: {str(e)}")

    def generate_secure_token(self, length: int = 32) -> str:
        """Generate a cryptographically secure token."""
        return secrets.token_urlsafe(length)

    def hash_file(self, file_path: str, algorithm: str = "sha256") -> str:
        """Calculate file hash using specified algorithm."""
        hash_obj = getattr(hashlib, algorithm)()
        with open(file_path, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b""):
                hash_obj.update(chunk)
        return hash_obj.hexdigest()

    def create_api_key(self, prefix: str = "sk") -> tuple:
        """Generate a secure API key with prefix."""
        raw_key = secrets.token_urlsafe(32)
        api_key = f"{prefix}_{raw_key}"
        hashed_key = hashlib.sha256(api_key.encode()).hexdigest()
        return api_key, hashed_key

    def verify_api_key(self, api_key: str, stored_hash: str) -> bool:
        """Verify an API key against its stored hash."""
        hashed_key = hashlib.sha256(api_key.encode()).hexdigest()
        return secrets.compare_digest(hashed_key, stored_hash)

    def generate_password(self, length: int = 16, include_special: bool = True) -> str:
        """
        Generate a secure password with specified requirements.
        
        Args:
            length: Length of the password
            include_special: Whether to include special characters
        """
        chars = string.ascii_letters + string.digits
        if include_special:
            chars += string.punctuation
        
        while True:
            password = ''.join(random.choice(chars) for _ in range(length))
            if (any(c.islower() for c in password) and
                any(c.isupper() for c in password) and
                any(c.isdigit() for c in password) and
                (not include_special or any(c in string.punctuation for c in password))):
                return password

    def validate_password_strength(self, password: str) -> Tuple[bool, List[str]]:
        """
        Validate password strength against security requirements.
        
        Returns:
            Tuple of (is_valid, list_of_failures)
        """
        failures = []
        if len(password) < 8:
            failures.append("Password must be at least 8 characters long")
        if not any(c.islower() for c in password):
            failures.append("Password must contain lowercase letters")
        if not any(c.isupper() for c in password):
            failures.append("Password must contain uppercase letters")
        if not any(c.isdigit() for c in password):
            failures.append("Password must contain digits")
        if not any(c in string.punctuation for c in password):
            failures.append("Password must contain special characters")
        
        return len(failures) == 0, failures

    def encrypt_asymmetric(self, data: str) -> str:
        """
        Encrypt data using RSA asymmetric encryption.
        
        Returns:
            Base64 encoded encrypted data
        """
        encrypted = self.public_key.encrypt(
            data.encode(),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )
        return base64.b64encode(encrypted).decode()

    def decrypt_asymmetric(self, encrypted_data: str) -> str:
        """
        Decrypt RSA encrypted data.
        
        Args:
            encrypted_data: Base64 encoded encrypted data
        """
        try:
            encrypted = base64.b64decode(encrypted_data)
            decrypted = self.private_key.decrypt(
                encrypted,
                padding.OAEP(
                    mgf=padding.MGF1(algorithm=hashes.SHA256()),
                    algorithm=hashes.SHA256(),
                    label=None
                )
            )
            return decrypted.decode()
        except Exception as e:
            raise ValueError(f"Asymmetric decryption failed: {str(e)}")

    def generate_key_pair(self) -> Tuple[str, str]:
        """
        Generate RSA key pair and return as PEM strings.
        
        Returns:
            Tuple of (private_key_pem, public_key_pem)
        """
        private_pem = self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ).decode()
        
        public_pem = self.public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        ).decode()
        
        return private_pem, public_pem

    def derive_key(self, password: str, salt: bytes = None) -> Tuple[bytes, bytes]:
        """
        Derive a key from a password using PBKDF2.
        
        Args:
            password: Password to derive key from
            salt: Optional salt bytes, generated if not provided
            
        Returns:
            Tuple of (key, salt)
            
        Raises:
            ValueError: If password is empty or invalid
        """
        if not password:
            raise ValueError("Password cannot be empty")
            
        try:
            salt = salt or os.urandom(16)
            kdf = PBKDF2HMAC(
                algorithm=hashes.SHA256(),
                length=32,
                salt=salt,
                iterations=100000,
            )
            key = kdf.derive(password.encode())
            return key, salt
        except Exception as e:
            raise ValueError(f"Key derivation failed: {str(e)}")

    def sanitize_input(self, input_str: str, allow_html: bool = False) -> str:
        """
        Sanitize input string to prevent XSS and injection attacks.
        
        Args:
            input_str: Input string to sanitize
            allow_html: Whether to allow safe HTML tags
            
        Returns:
            Sanitized string
        """
        if not isinstance(input_str, str):
            raise ValueError("Input must be a string")
            
        if not input_str:
            return ""
            
        # First pass: remove all HTML tags if not allowed
        if not allow_html:
            input_str = re.sub(r'<[^>]*>', '', input_str)
        
        # Second pass: escape special characters
        input_str = input_str.replace('&', '&amp;')
        input_str = input_str.replace('<', '&lt;')
        input_str = input_str.replace('>', '&gt;')
        input_str = input_str.replace('"', '&quot;')
        input_str = input_str.replace("'", '&#x27;')
        input_str = input_str.replace('/', '&#x2F;')
        input_str = input_str.replace('\\', '&#x5C;')
        
        # Third pass: remove potentially dangerous patterns
        input_str = re.sub(r'javascript:', '', input_str, flags=re.IGNORECASE)
        input_str = re.sub(r'data:', '', input_str, flags=re.IGNORECASE)
        input_str = re.sub(r'vbscript:', '', input_str, flags=re.IGNORECASE)
        
        return input_str

    def rate_limit_check(self, key: str, max_requests: int, time_window: int) -> Tuple[bool, int]:
        """
        Check if request should be rate limited.
        
        Args:
            key: Identifier for the rate limit (e.g., IP address)
            max_requests: Maximum number of requests allowed
            time_window: Time window in seconds
            
        Returns:
            Tuple of (is_allowed, remaining_requests)
            
        Note:
            This is a basic in-memory implementation. For production use,
            implement this using Redis or a similar distributed cache.
        """
        if not hasattr(self, '_rate_limit_store'):
            self._rate_limit_store = {}
            
        current_time = int(datetime.utcnow().timestamp())
        
        # Clean up expired entries
        if key in self._rate_limit_store:
            requests = [(ts, count) for ts, count in self._rate_limit_store[key] 
                       if current_time - ts < time_window]
        else:
            requests = []
            
        # Count total requests in window
        total_requests = sum(count for _, count in requests)
        
        # Update store
        if total_requests < max_requests:
            requests.append((current_time, 1))
            self._rate_limit_store[key] = requests
            return True, max_requests - total_requests - 1
            
        return False, 0

    def validate_url(self, url: str, allowed_schemes: List[str] = None) -> bool:
        """
        Validate URL for security concerns.
        
        Args:
            url: URL to validate
            allowed_schemes: List of allowed URL schemes
        """
        if allowed_schemes is None:
            allowed_schemes = ['http', 'https']
            
        try:
            parsed = urllib.parse.urlparse(url)
            return (
                parsed.scheme in allowed_schemes and
                parsed.netloc and
                not any(c in url for c in '<>"\'{}|\\^`')
            )
        except Exception:
            return False

    def validate_ip(self, ip: str, allowed_ranges: List[str] = None) -> bool:
        """
        Validate IP address and check if it's in allowed ranges.
        
        Args:
            ip: IP address to validate
            allowed_ranges: List of allowed IP ranges in CIDR notation
        """
        try:
            ip_obj = ipaddress.ip_address(ip)
            if allowed_ranges is None:
                return True
                
            return any(
                ip_obj in ipaddress.ip_network(cidr)
                for cidr in allowed_ranges
            )
        except ValueError:
            return False

    def generate_nonce(self, length: int = 16) -> str:
        """Generate a secure nonce for cryptographic operations."""
        return secrets.token_hex(length)

    def hash_data(self, data: Union[str, bytes], algorithm: str = "sha256") -> str:
        """
        Hash data using specified algorithm.
        
        Args:
            data: Data to hash (string or bytes)
            algorithm: Hash algorithm to use
        """
        hash_obj = getattr(hashlib, algorithm)()
        if isinstance(data, str):
            data = data.encode()
        hash_obj.update(data)
        return hash_obj.hexdigest()

    def generate_session_id(self) -> str:
        """Generate a secure session ID."""
        return f"sess_{secrets.token_urlsafe(32)}"

    def validate_file_type(self, filename: str, allowed_extensions: List[str]) -> bool:
        """
        Validate file type based on extension.
        
        Args:
            filename: Name of the file
            allowed_extensions: List of allowed file extensions
        """
        return any(
            filename.lower().endswith(ext.lower())
            for ext in allowed_extensions
        )

    def generate_backup_code(self, length: int = 8, num_codes: int = 10) -> List[str]:
        """
        Generate backup codes for 2FA recovery.
        
        Args:
            length: Length of each code
            num_codes: Number of codes to generate
        """
        codes = set()
        while len(codes) < num_codes:
            code = ''.join(
                random.choice(string.ascii_uppercase + string.digits)
                for _ in range(length)
            )
            codes.add(code)
        return sorted(list(codes))



from moviepy.editor import ImageClip, concatenate_videoclips
from moviepy.video.fx import all as vfx
import os

class VideosCreator:
    """
    A class for creating videos from images using MoviePy.

    Nested Class:
    - Basic: Provides basic functionalities for video creation.

    Methods:
    - basic_video_creator: Creates a video from images with basic effects and options.

    Attributes:
    - VIDEO_DURATIONS: Dictionary mapping video platforms to their maximum durations.
    """
    
    @staticmethod
    class Basic:
        """
        Provides basic functionalities for creating videos from images.
        """
        
        def __init__(self):
            pass

        @staticmethod
        def basic_video_creator(image_folder="images/", animation_choice="None", frame_rate=25, video_name="output", video_type="mp4", video_platform="Youtube", image_time=5):
            """
            Creates a video from images with specified parameters.

            Args:
            - image_folder: Folder containing images.
            - animation_choice: Animation effect between images (FadeIn, FadeOut, Rotate, FlipHorizontal, FlipVertical).
            - frame_rate: Frames per second for the video.
            - video_name: Name of the output video file.
            - video_type: Type of the output video file (e.g., mp4).
            - video_platform: Platform for which the video is optimized (Youtube, Facebook, Instagram, Tiktok).
            - image_time: Duration each image appears in seconds.

            Returns:
            - 'done' if video creation is successful.
            """

            VIDEO_DURATIONS = {
                'Youtube': 60,
                'Facebook': 20,
                'Instagram': 15,
                'Tiktok': 60
            }

            try:
                files = os.listdir(image_folder)
                image_files = [os.path.join(image_folder, f) for f in files if f.lower().endswith(('.jpg', '.jpeg', '.png'))]
                image_files.sort()
            except Exception as error:
                raise error

            if video_platform in VIDEO_DURATIONS:
                video_duration = VIDEO_DURATIONS[video_platform]
            else:
                raise ValueError(f"Unsupported video platform: {video_platform}. Choose from Youtube, Facebook, Instagram, or Tiktok.")

            video_clips = []
            for i, image_file in enumerate(image_files):
                clip = ImageClip(image_file).set_duration(image_time)
                video_clips.append(clip)
                
                if i < len(image_files) - 1 and animation_choice:
                    next_clip = ImageClip(image_files[i + 1]).set_duration(image_time)
                    if animation_choice == 'FadeIn':
                        fade_duration = min(1, image_time / 2)
                        video_clips.append(next_clip.crossfadein(fade_duration).set_start(clip.end))
                    elif animation_choice == 'FadeOut':
                        video_clips.append(clip.crossfadeout(1).set_end(clip.end))
                    elif animation_choice == 'Rotate':
                        rotate_clip = next_clip.rotate(lambda t: 360*t).set_start(clip.end)
                        video_clips.append(rotate_clip)
                    elif animation_choice == 'FlipHorizontal':
                        video_clips.append(next_clip.fx(vfx.mirror_x).set_start(clip.end))
                    elif animation_choice == 'FlipVertical':
                        video_clips.append(next_clip.fx(vfx.mirror_y).set_start(clip.end))
            
            final_video = concatenate_videoclips(video_clips, method="compose")
            
            output_file = f"{video_name}.{video_type}"
            final_video.write_videofile(output_file, fps=frame_rate)
            return 'done'
        



import platform
import os
import subprocess
   
class System:
    """
    A class for managing system operations.
    """
    
    @staticmethod
    def hibernate():
        """
        Hibernate the system.

        Raises:
        - NotImplementedError: If the OS is not supported (only Windows is supported).
        """
        system = platform.system()
        if system == "Windows":
            os.system("shutdown /h")
        else:
            raise NotImplementedError("Unsupported OS")

    @staticmethod
    def restart():
        """
        Restart the system.

        Raises:
        - NotImplementedError: If the OS is not supported (only Windows is supported).
        """
        system = platform.system()
        if system == "Windows":
            os.system("shutdown /r /t 1")
        else:
            raise NotImplementedError("Unsupported OS")

    @staticmethod
    def shutdown():
        """
        Shutdown the system.

        Raises:
        - NotImplementedError: If the OS is not supported (Windows, Linux, and macOS are supported).
        """
        system = platform.system()
        if system == "Windows":
            subprocess.run(["shutdown", "/s", "/t", "1"])
        elif system == "Linux" or system == "Darwin":
            subprocess.run(["sudo", "shutdown", "-h", "now"])
        else:
            raise NotImplementedError("Unsupported OS")
        
    @staticmethod
    def log_off():
        """
        Log off the current user.

        Raises:
        - NotImplementedError: If the OS is not supported (only Windows is supported).
        """
        system = platform.system()
        if system == "Windows":
            os.system("shutdown /l")
        else:
            raise NotImplementedError("Unsupported OS")



import os

class Libraries:
    
    @staticmethod
    class Basic:
        
        def __init__(self):
            pass
        
        @staticmethod
        def init_creator(filesave="__init__.py", filename="", function_class=""):
            """
            Initializes a Python file with import statements.

            Args:
            - filesave: File path to save the initialization.
            - filename: Name of the Python file to import from.
            - function_class: Name of the function or class to import.

            Returns:
            - 'done' if successful.
            - Error message if unsuccessful.
            """
            if filename == "" or function_class == "" or filesave == "":
                return "FileSave or FileName or Function/Class Name is Not Found"
            
            try:
                if os.path.exists(filesave):
                    with open(filesave, "r") as f:
                        text = f.read()
                else:
                    text = ""

                text += f"\nfrom .{filename} import {function_class}\n"
                
                with open(filesave, "w") as f:
                    f.write(text)
                
                return 'done'
            except Exception as e:
                return str(e)

        @staticmethod
        def basic_setup_file_creator(filename="setup.py", folder_name="", readme_name="README.md", library_name="", library_version="", libraries_required=[], description="", creator_name="", creator_email="", License="MIT"):
            """
            Creates a basic setup.py file for a Python library.

            Args:
            - filename: Name of the setup file to create.
            - folder_name: Folder name (not used in function logic).
            - readme_name: Name of the README file.
            - library_name: Name of the Python library.
            - library_version: Version of the Python library.
            - libraries_required: List of required libraries.
            - description: Description of the Python library.
            - creator_name: Name of the library creator.
            - creator_email: Email of the library creator.
            - License: License type (default: MIT).

            Returns:
            - 'done' if successful.
            - 'FileName Found' if filename already exists.
            - Error message if unsuccessful.
            """
            if License == "MIT":
                libraries_required.append("YWP")
                file_data = (
                    "from setuptools import setup, find_packages\n\n"
                    f"setup(\nname='{library_name}',\nversion='{library_version}',\n"
                    f"packages=find_packages(),\ninstall_requires={str(libraries_required)},\n"
                    "classifiers=[\n'Programming Language :: Python :: 3',\n],\n"
                    "python_requires='>=3.6',\ndescription='" + description + "',\n"
                    f"long_description=open('{readme_name}').read(),\n"
                    "long_description_content_type='text/markdown',\n"
                    f"author='{creator_name}',\nauthor_email='{creator_email}',\n"
                    ")"
                )
                
                if os.path.exists(filename):
                    return 'FileName Found'
                
                try:
                    with open(filename, "w") as f:
                        f.write(file_data)
                    return 'done'
                except Exception as e:
                    return str(e)
            else:
                return 'Not From Licenses'

        @staticmethod
        def upload_file_creator(filename="upload_library", pypi_api="", platform="windows"):
            """
            Creates upload scripts for distributing a Python library.

            Args:
            - filename: Name of the upload script file.
            - pypi_api: PyPI API key or token.
            - platform: Platform to generate script for (windows or linux).

            Returns:
            - 'done' if successful.
            - 'FileName Found' if filename already exists.
            - 'Platform Not Supported' if platform is not windows or linux.
            - Error message if unsuccessful.
            """
            platforms = ["windows", "linux"]
            
            if platform in platforms:
                if platform == "windows":
                    filename += ".bat"
                    file_data = (
                        "set TWINE_USERNAME=__token__\n"
                        f"set TWINE_PASSWORD={pypi_api}\n"
                        "python setup.py sdist bdist_wheel\n"
                        "set TWINE_USERNAME=%TWINE_USERNAME% "
                        "set TWINE_PASSWORD=%TWINE_PASSWORD% "
                        "twine upload dist/*"
                    )
                elif platform == "linux":
                    filename += ".sh"
                    file_data = (
                        'export TWINE_USERNAME="__token__"\n'
                        f'export TWINE_PASSWORD="{pypi_api}"\n'
                        'python setup.py sdist bdist_wheel\n'
                        'TWINE_USERNAME="$TWINE_USERNAME" '
                        'TWINE_PASSWORD="$TWINE_PASSWORD" '
                        'twine upload dist/*'
                    )
                
                if os.path.exists(filename):
                    return 'FileName Found'
                
                try:
                    with open(filename, "w") as f:
                        f.write(file_data)
                    return 'done'
                except Exception as e:
                    return str(e)
            else:
                return 'Platform Not Supported'



import requests
import openai

class AI:
    """
    Contains all AI related functions.
    """

    class Gemini:
        """
        Interacts with the Gemini AI API.
        """

        def __init__(self, api_key: str, base_url="https://api.gemini.ai/v1"):
            """
            Initializes the Gemini AI Class.

            Args:
            - api_key (str): The API Key for the Gemini AI API.
            """
            self.api_key = api_key
            self.base_url = base_url

        def send_message(self, message: str):
            """
            Sends a message to the Gemini AI API.

            Args:
            - message (str): The message to send.

            Returns:
            - str: The response from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/text"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'input': message
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def get_conversations(self):
            """
            Gets all conversations from the Gemini AI API.

            Returns:
            - list: A list of conversations from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.get(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def get_conversation(self, conversation_id: str):
            """
            Gets a conversation by ID from the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to get.

            Returns:
            - dict: The conversation from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.get(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def create_conversation(self, message: str):
            """
            Creates a new conversation on the Gemini AI API.

            Args:
            - message (str): The initial message for the conversation.

            Returns:
            - dict: The created conversation from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'input': message
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def reply_to_conversation(self, conversation_id: str, message: str):
            """
            Replies to a conversation on the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to reply to.
            - message (str): The message to send.

            Returns:
            - dict: The updated conversation from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}/messages"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'input': message
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def get_messages(self, conversation_id: str):
            """
            Gets all messages from a conversation on the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to get messages from.

            Returns:
            - list: A list of messages from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}/messages"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.get(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def get_message(self, conversation_id: str, message_id: str):
            """
            Gets a message by ID from the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to get the message from.
            - message_id (str): The ID of the message to get.

            Returns:
            - dict: The message from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}/messages/{message_id}"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.get(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def delete_conversation(self, conversation_id: str):
            """
            Deletes a conversation from the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to delete.

            Returns:
            - dict: The deleted conversation from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.delete(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def delete_message(self, conversation_id: str, message_id: str):
            """
            Deletes a message from the Gemini AI API.

            Args:
            - conversation_id (str): The ID of the conversation to delete the message from.
            - message_id (str): The ID of the message to delete.

            Returns:
            - dict: The deleted message from the Gemini AI API.
            """
            try:
                url = f"{self.base_url}/conversations/{conversation_id}/messages/{message_id}"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                response = requests.delete(url, headers=headers)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

    class ChatGPT:
        """
        Interacts with the OpenAI ChatGPT API.
        """

        def __init__(self, api_key: str, model: str = "gpt-4"):
            """
            Initializes the ChatGPT Class.

            Args:
            - api_key (str): The API Key for the OpenAI API.
            - model (str): The model to use (default: "gpt-4").
            """
            openai.api_key = api_key
            self.model = model

        def send_message(self, message: str, system_prompt: str = None):
            """
            Sends a message to ChatGPT.

            Args:
            - message (str): The message to send.
            - system_prompt (str): Optional system prompt to set context.

            Returns:
            - dict: The response from ChatGPT.
            """
            try:
                messages = []
                if system_prompt:
                    messages.append({"role": "system", "content": system_prompt})
                messages.append({"role": "user", "content": message})
                
                response = openai.ChatCompletion.create(
                    model=self.model,
                    messages=messages
                )
                return response
            except Exception as e:
                print(f"An error occurred: {e}")
                return None

        def create_chat(self, messages: list):
            """
            Creates a chat with multiple messages.

            Args:
            - messages (list): List of message dictionaries with 'role' and 'content'.

            Returns:
            - dict: The chat completion response.
            """
            try:
                response = openai.ChatCompletion.create(
                    model=self.model,
                    messages=messages
                )
                return response
            except Exception as e:
                print(f"An error occurred: {e}")
                return None

        def stream_chat(self, messages: list):
            """
            Creates a streaming chat response.

            Args:
            - messages (list): List of message dictionaries.

            Returns:
            - generator: Stream of response chunks.
            """
            try:
                response = openai.ChatCompletion.create(
                    model=self.model,
                    messages=messages,
                    stream=True
                )
                return response
            except Exception as e:
                print(f"An error occurred: {e}")
                return None

    class DeepSeek:
        """
        Interacts with the DeepSeek AI API.
        """

        def __init__(self, api_key: str, base_url: str = "https://api.deepseek.com/v1"):
            """
            Initializes the DeepSeek AI Class.

            Args:
            - api_key (str): The API Key for the DeepSeek API.
            - base_url (str): The base URL for the API.
            """
            self.api_key = api_key
            self.base_url = base_url

        def generate_text(self, prompt: str, max_tokens: int = 100):
            """
            Generates text using DeepSeek AI.

            Args:
            - prompt (str): The input prompt.
            - max_tokens (int): Maximum number of tokens to generate.

            Returns:
            - dict: The generated text response.
            """
            try:
                url = f"{self.base_url}/completions"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'prompt': prompt,
                    'max_tokens': max_tokens
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def generate_code(self, prompt: str, language: str = "python"):
            """
            Generates code using DeepSeek AI.

            Args:
            - prompt (str): The input prompt.
            - language (str): The programming language.

            Returns:
            - dict: The generated code response.
            """
            try:
                url = f"{self.base_url}/code/completions"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'prompt': prompt,
                    'language': language
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None

        def analyze_code(self, code: str):
            """
            Analyzes code using DeepSeek AI.

            Args:
            - code (str): The code to analyze.

            Returns:
            - dict: The analysis response.
            """
            try:
                url = f"{self.base_url}/code/analyze"
                headers = {
                    'Authorization': f'Bearer {self.api_key}',
                    'Content-Type': 'application/json'
                }
                data = {
                    'code': code
                }
                response = requests.post(url, headers=headers, json=data)
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")
                return None


from flask import Flask

class Server:
    """
    A class for creating and running a Flask server.

    Attributes:
    - app: Holds the Flask application instance.
    """

    def __init__(self):
        """
        Initializes the server instance with None.
        """
        self.apps = None

    def route_flask(self, location="", returnValue=""):
        """
        Adds a route to the Flask application.

        Args:
        - location: URL endpoint for the route.
        - returnValue: Value returned by the route function.

        Returns:
        - 'done' if route addition is successful.
        """
        app = self.apps
        try:
            if app is None:
                app = Flask(__name__)

            def make_route(return_value):
                def route():
                    return return_value
                return route

            endpoint = location.strip('/')
            if endpoint == '':
                endpoint = 'index'

            app.add_url_rule(location, endpoint, make_route(returnValue))
            apps = app
            return 'done'
        except Exception as error:
            raise error
        
    def run(self, check=False, debug=True, host="0.0.0.0", port="8000"):
        """
        Starts the Flask server.

        Args:
        - check: If True, runs only if __name__ == "__main__".
        - debug: Enables debug mode if True.
        - host: Host IP address to run the server on.
        - port: Port number to run the server on.

        Returns:
        - 'done' if server starts successfully.
        """
        app = self.apps
        try:
            if app is None:
                raise Exception("App not initialized")
            
            if check:
                if __name__ == "__main__":
                    app.run(debug=debug, host=host, port=port)
            else:
                app.run(debug=debug, host=host, port=port)
            return 'done'
        except Exception as error:
            raise error



"""
INEX Library
============

A comprehensive Python library for AI integration, web development, encryption,
database management, file operations, and more.

Package Name: inex-library
Contact Information:
------------------
- Phone/WhatsApp: +20109673019
- Email: pbstzidr@ywp.freewebhostmost.com
- GitHub: https://github.com/AmmarBasha2011/inex_library
- PyPI: https://pypi.org/project/inex-library

Version: 1.0.0
Last Updated: 2025-01-20
"""

__version__ = '1.0.0'
__author__ = 'INEX Team'
__email__ = 'pbstzidr@ywp.freewebhostmost.com'
__url__ = 'https://github.com/AmmarBasha2011/inex_library'
__description__ = 'A comprehensive Python library for AI integration, web development, and more.'
__copyright__ = '2025 INEX Team'
__license__ = 'MIT'

from . import AI
from . import Crypto
from . import Database
from . import EnDeCrypt
from . import FastAPIServer
from . import Files
from . import Libraries
from . import PrintStyle
from . import Security
from . import Server
from . import System
from . import Translate
from . import VideosCreator
from . import Websites

__all__ = [
    'AI',
    'Crypto',
    'Database',
    'Files',
    'Libraries',
    'System',
    'Translate',
    'VideosCreator',
    'Websites',
    'EnDeCrypt',
    'FastAPIServer',
    'PrintStyle',
    'Security',
    'Server'
]



import os
import subprocess

class Files:
    
    @staticmethod
    def delete_file(filepath):
        """
        Deletes a file if it exists.

        Args:
        - filepath: Path to the file to delete.

        Returns:
        - "Deleted" if file is successfully deleted.
        - Raises an exception if deletion fails.
        """
        if os.path.exists(filepath):
            try:
                os.remove(filepath)
                return "Deleted"
            except Exception as error:
                raise error

    @staticmethod
    def open_file(filepath=""):
        """
        Opens a file if it exists using subprocess.

        Args:
        - filepath: Path to the file to open.

        Returns:
        - "open" if file is successfully opened.
        - "Not Found Path" if file path does not exist.
        - "An error occurred" with the error message if an exception occurs.
        """
        try:
            if os.path.exists(filepath):
                subprocess.Popen([str(filepath)])
                return "open"
            else:
                return "Not Found Path"
        except Exception as e:
            print("An error occurred:", e)
            return "An error occurred:", e

    @staticmethod
    def create_file(name=""):
        """
        Creates a new file and writes user input into it.

        Args:
        - name: Name of the file to create.

        Returns:
        - "created" if file is successfully created and written to.
        - Raises an exception if creation fails.
        """
        print("Please enter the text or code (press Ctrl + D on Unix or Ctrl + Z then Enter on Windows to finish):")

        user_input_lines = []
        try:
            while True:
                line = input()
                user_input_lines.append(line)
        except EOFError:
            pass

        user_input = '\n'.join(user_input_lines)
        
        filename = name

        try:
            with open(filename, "w", encoding="utf-8") as file:
                file.write(user_input)
            return "created"
        except Exception as error:
            raise error

    @staticmethod
    def delete_all_files(directory=".", type={}):
        """
        Deletes files in a directory based on specified types.

        Args:
        - directory: Directory path where files are located.
        - type: Dictionary mapping index to file types to delete.

        Returns:
        - "Deleted" if files are successfully deleted.
        - Raises an exception if deletion fails.
        """
        
        for filename in os.listdir(directory):
            for index, filetype in type.items():
                if filename.endswith(filetype):
                    filepath = os.path.join(directory, filename)
                    try:
                        os.remove(filepath)
                    except Exception as error:
                        raise error
        return "Deleted"


from fastapi import FastAPI
from typing import Callable

class FastAPIServer:
    """
    A class to create a FastAPI server.
    """

    def __init__(self):
        """
        Initializes a FastAPI server.
        """
        self.app = FastAPI()

    def add_endpoint(self, path: str, function: Callable, method: str = "GET"):
        """
        Adds a new endpoint to the server.

        Args:
        - path (str): The path of the endpoint.
        - function (Callable): The function to be called when the endpoint is accessed.
        - method (str): The HTTP method to use (default: "GET").
        """
        self.app.add_api_route(path=path, methods=[method], endpoint=lambda request: function(**request.query_params))

    def add_cors(self):
        """
        Adds CORS support to the server.
        """
        from fastapi.middleware.cors import CORSMiddleware

        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )

    def run(self, host: str = "0.0.0.0", port: int = 8000):
        """
        Starts the FastAPI server.

        Args:
        - host (str): The host to listen to (default: "0.0.0.0").
        - port (int): The port to listen to (default: 8000).
        """
        import uvicorn
        uvicorn.run(self.app, host=host, port=port)


import sqlite3

class Database:
    '''
    Class for creating and managing SQLite databases.

    Attributes:
    - connection: sqlite3 connection object
    - database_name: name of the database file

    Methods:
    - __init__: Initializes the database with a name and optional password.
    - create_table: Creates a table in the database with the given name and columns.
    - fetch: Fetches data from the database table based on the given conditions.

    Notes:
    - If the password is not provided, the database will not be encrypted.
    - The password should be kept confidential to ensure the security of the database.
    '''
    
    def __init__(self, database_name, password=None):
        '''
        Initializes the database with a name and optional password.

        Args:
        - database_name (str): Name of the database file.
        - password (str): Password for encrypting the database. If not provided, the database will not be encrypted.

        Returns:
        - A Database object.
        '''
        
        try:
            if password:
                self.connection = sqlite3.connect(f'{database_name}?key={password}', uri=True)
            else:
                self.connection = sqlite3.connect(database_name)
            self.connection.row_factory = sqlite3.Row
            self.database_name = database_name
        except sqlite3.Error as e:
            print(f"An error occurred: {e}")
            self.connection = None
    
    def create_table(self, table_name, columns=[{"id": "INTEGER PRIMARY KEY AUTOINCREMENT"}]):
        '''
        Creates a table in the database with the given name and columns.

        Args:
        - table_name (str): Name of the table to create.
        - columns (list): List of dictionaries containing the column names and types. Each dictionary should have the column name as the key and the column type as the value.

        Returns:
        - True if the table was created successfully, False otherwise.

        Notes:
        - The table will be created with the given columns and an "id" column as the primary key with auto increment.
        - The columns should be a list of dictionaries, where each dictionary contains the column name and type.
        '''
        
        try:
            cursor = self.connection.cursor()
            cursor.execute(f'CREATE TABLE IF NOT EXISTS {table_name} ({", ".join([f"{column_name} {column_type}" for column_name, column_type in columns.items()])})')
            return True
        except sqlite3.Error as e:
            print(f"An error occurred: {e}")
            return False
        
    def fetch(self, table_name, columns=["*"], where=[None], type="all"):
        '''
        Fetches data from the database table based on the given conditions.

        Args:
        - table_name (str): Name of the table to fetch from.
        - columns (list): List of column names to fetch. If not provided, all columns will be fetched.
        - where (list): List of conditions to filter the data. Each condition should be a string with the column name, operator, and value.
        - type (str): Type of fetch to perform. If "one", only one row will be fetched. If "all", all rows will be fetched.

        Returns:
        - If type is "one", a single row as a dictionary. If type is "all", a list of dictionaries containing all the rows.

        Notes:
        - The where parameter should be a list of strings with the column name, operator, and value.
        - The type parameter should be "one" or "all".
        '''
        
        try:
            if type == "one":
                cursor = self.connection.cursor()
                cursor.execute(f'SELECT {", ".join(columns)} FROM {table_name} WHERE {" AND ".join(where)}', (where[1],))
                return cursor.fetchone()
            elif type == "all":
                cursor = self.connection.cursor()
                cursor.execute(f'SELECT {", ".join(columns)} FROM {table_name} WHERE {" AND ".join(where)}', (where[1],))
                return cursor.fetchall()
            else:
                print("Invalid type")
                return []
        except sqlite3.Error as e:
            print(f"An error occurred: {e}")
            return []


from setuptools import setup, find_packages

setup(
    name='inex-library',
    version='1.0.0',
    packages=find_packages(),
    install_requires=[
        'fastapi>=0.68.0',  # Core FastAPI framework
        'uvicorn>=0.15.0',  # ASGI server for FastAPI
        'starlette>=0.14.0',  # Web framework toolkit
        'pydantic>=1.8.0',  # Data validation
        'flask>=2.0.0',  # Flask web framework
        'pycryptodome>=3.10.0',  # For AES and Blowfish encryption
        'cryptography>=3.4.0',  # For additional cryptographic operations
        'requests>=2.26.0',  # HTTP requests
        'openai>=1.0.0',
        'python-jose>=3.3.0',  # For JWT handling
        'passlib>=1.7.4',  # For password hashing
        'pyjwt>=2.1.0',  # For JWT operations
        'pynacl>=1.4.0',  # For cryptographic signing
        'python-multipart>=0.0.5',  # For handling form data in FastAPI
        'python-dotenv>=0.19.0',
        'aiofiles>=0.7.0',  # For async file operations
        'watchdog>=2.1.0',  # For file system monitoring
        'pathlib>=1.0.1',  # For path manipulation
        'pyotp>=2.6.0',  # For 2FA operations
        'python-ipaddress>=1.0.0',  # For IP validation
        'moviepy>=1.0.3',  # For video creation and editing
        'googletrans>=3.1.0a0',  # For text translation
        'pillow>=8.3.0',  # For image processing
        'web3>=5.24.0',  # For blockchain interactions
        'sqlalchemy>=1.4.0',  # For advanced database operations
        'alembic>=1.7.0',  # For database migrations
        'sqlcipher3>=0.4.5',  # For database encryption
        'twine>=3.4.2',  # For PyPI package distribution
        'wheel>=0.37.0',  # For building wheel distributions
        'setuptools>=57.0.0',  # For package setup
        'build>=0.7.0',  # For building Python packages
        'colorama>=0.4.4',  # For colored terminal output
        'termcolor>=1.1.0',  # For terminal text styling
        'rich>=10.12.0',  # For rich text and formatting
        'bcrypt>=3.2.0',  # For password hashing
        'pyOpenSSL>=20.0.0',  # For SSL/TLS operations
        'cryptography-vectors>=3.4.0',  # Test vectors for cryptography
        'python-magic>=0.4.24',  # For file type detection
        'redis>=4.0.0',  # For rate limiting and caching
        'bleach>=4.1.0',  # For input sanitization
        'validators>=0.18.0',  # For URL validation
        'gunicorn>=20.1.0',  # Production WSGI server
        'hypercorn>=0.11.0',  # Alternative ASGI server
        'werkzeug>=2.0.0',  # WSGI utilities
        'click>=8.0.0',  # Command line interface
        'jinja2>=3.0.0',  # Template engine
        'itsdangerous>=2.0.0',  # Data signing
        'psutil>=5.8.0',  # System and process utilities
        'pywin32>=300; platform_system == "Windows"',  # Windows system utilities
        'pyuac>=0.0.3; platform_system == "Windows"',  # Windows UAC support
        'elevate>=0.1.3',  # Cross-platform privilege elevation
        'opencv-python>=4.5.0',  # For video processing
        'numpy>=1.21.0',  # For numerical operations
        'ffmpeg-python>=0.2.0',  # For video encoding/decoding
        'beautifulsoup4>=4.9.0',  # For web scraping
        'selenium>=4.0.0',  # For web automation
        'playwright>=1.20.0',  # Modern web automation
        'urllib3>=1.26.0',  # HTTP client
        'aiohttp>=3.8.0',  # Async HTTP client
        'httpx>=0.23.0',  # Modern HTTP client
    ],
    extras_require={
        'dev': [
            'pytest>=6.2.0',
            'black>=21.7b0',
            'isort>=5.9.0',
            'flake8>=3.9.0',
            'httpx>=0.18.0',  # For FastAPI testing
            'pytest-asyncio>=0.15.0',  # For async tests
            'safety>=1.10.0',  # For security vulnerability checking
            'bandit>=1.7.0',  # For security linting
            'locust>=2.8.0',  # For load testing
            'coverage>=6.2',  # For code coverage
            'pytest-cov>=3.0.0',  # For pytest coverage
        ],
        'database': [
            'psycopg2-binary>=2.9.0',  # PostgreSQL support
            'pymysql>=1.0.0',  # MySQL support
            'sqlcipher3>=0.4.5',  # Database encryption
            'cryptography>=3.4.0',  # For database encryption
        ],
        'crypto': [
            'pyOpenSSL>=20.0.0',  # For SSL/TLS operations
            'cryptography-vectors>=3.4.0',  # Test vectors for cryptography
            'bcrypt>=3.2.0',  # For password hashing
            'pynacl>=1.4.0',  # For cryptographic signing
            'pyotp>=2.6.0',  # For 2FA operations
            'qrcode>=7.3.0',  # For 2FA QR codes
        ],
        'server': [
            'gunicorn>=20.1.0',  # Production WSGI server
            'hypercorn>=0.11.0',  # Alternative ASGI server
            'fastapi-utils>=0.2.0',  # Additional FastAPI utilities
            'fastapi-jwt-auth>=0.5.0',  # JWT authentication
            'fastapi-cache>=0.1.0',  # Response caching
            'fastapi-limiter>=0.1.5',  # Rate limiting
            'secure>=0.3.0',  # Security headers
            'flask-cors>=3.0.0',  # CORS support for Flask
            'flask-compress>=1.10.0',  # Response compression
            'flask-caching>=1.10.0',  # Flask caching
            'flask-limiter>=2.5.0',  # Flask rate limiting
            'flask-talisman>=0.8.0',  # Security headers for Flask
            'flask-seasurf>=0.3.0',  # CSRF protection
            'prometheus-flask-exporter>=0.18.0',  # Metrics
            'sentry-sdk>=1.5.0',  # Error tracking
            'supervisor>=4.2.0',  # Process management
            'nginx>=1.20.0',  # Reverse proxy
        ],
        'files': [
            'watchdog>=2.1.0',  # File system monitoring
            'pathlib>=1.0.1',  # Path manipulation
            'send2trash>=1.8.0',  # Safe file deletion
            'python-magic>=0.4.24',  # File type detection
            'aiofiles>=0.7.0',  # Async file operations
            'filetype>=1.0.0',  # File type validation
        ],
        'packaging': [
            'twine>=3.4.2',  # PyPI package distribution
            'wheel>=0.37.0',  # Wheel distribution building
            'build>=0.7.0',  # Python package building
            'check-manifest>=0.47.0',  # Package manifest verification
            'readme-renderer>=29.0',  # README rendering for PyPI
            'keyring>=23.0.1',  # Secure credential storage
        ],
        'styling': [
            'colorama>=0.4.4',  # Colored terminal output
            'termcolor>=1.1.0',  # Terminal text styling
            'rich>=10.12.0',  # Rich text and formatting
            'blessed>=1.19.0',  # Terminal styling
            'prompt-toolkit>=3.0.20',  # Interactive prompts
        ],
        'security': [
            'cryptography>=3.4.0',  # Core cryptography
            'pycryptodome>=3.10.0',  # Additional crypto algorithms
            'python-jose>=3.3.0',  # JWT handling
            'passlib>=1.7.4',  # Password hashing
            'bcrypt>=3.2.0',  # Password hashing
            'pyotp>=2.6.0',  # 2FA
            'qrcode>=7.3.0',  # 2FA QR codes
            'redis>=4.0.0',  # Rate limiting
            'bleach>=4.1.0',  # Input sanitization
            'validators>=0.18.0',  # URL validation
            'python-ipaddress>=1.0.0',  # IP validation
            'secure>=0.3.0',  # Security headers
            'pyOpenSSL>=20.0.0',  # SSL/TLS
            'safety>=1.10.0',  # Vulnerability checking
            'bandit>=1.7.0',  # Security linting
        ],
        'system': [
            'psutil>=5.8.0',  # System and process utilities
            'pywin32>=300; platform_system == "Windows"',  # Windows system utilities
            'pyuac>=0.0.3; platform_system == "Windows"',  # Windows UAC support
            'elevate>=0.1.3',  # Cross-platform privilege elevation
            'distro>=1.6.0',  # Linux distribution info
            'py-cpuinfo>=8.0.0',  # CPU information
            'gputil>=1.4.0',  # GPU utilities
            'psutil>=5.8.0',  # Process and system utilities
            'pyusb>=1.2.0',  # USB device management
            'pyserial>=3.5',  # Serial port communication
            'netifaces>=0.11.0',  # Network interfaces
            'ifaddr>=0.1.7',  # Network interface addresses
        ],
        'translation': [
            'googletrans>=3.1.0a0',  # Google Translate API
            'translate>=3.6.1',  # Translation utilities
            'deep-translator>=1.8.3',  # Multiple translation services
            'langdetect>=1.0.9',  # Language detection
            'polyglot>=16.7.4',  # Natural language processing
            'pycld2>=0.41',  # Compact Language Detector
            'langid>=1.1.6',  # Language identification
            'mtranslate>=1.8',  # Microsoft Translator API
            'textblob>=0.17.1',  # Text processing and translation
            'lingua>=4.15.0',  # Language detection
        ],
        'video': [
            'moviepy>=1.0.3',  # Video editing
            'opencv-python>=4.5.0',  # Video processing
            'numpy>=1.21.0',  # Numerical operations
            'ffmpeg-python>=0.2.0',  # Video encoding/decoding
            'pillow>=8.3.0',  # Image processing
            'imageio>=2.9.0',  # Image I/O
            'imageio-ffmpeg>=0.4.5',  # FFmpeg support
            'scikit-image>=0.18.0',  # Image processing
            'scipy>=1.7.0',  # Scientific computing
            'av>=9.0.0',  # PyAV for fast video processing
            'python-vlc>=3.0.0',  # VLC bindings
            'pims>=0.5.0',  # Video frame access
        ],
        'web': [
            'beautifulsoup4>=4.9.0',  # Web scraping
            'selenium>=4.0.0',  # Web automation
            'playwright>=1.20.0',  # Modern web automation
            'requests>=2.26.0',  # HTTP requests
            'urllib3>=1.26.0',  # HTTP client
            'aiohttp>=3.8.0',  # Async HTTP client
            'httpx>=0.23.0',  # Modern HTTP client
            'html5lib>=1.1',  # HTML parser
            'lxml>=4.9.0',  # XML/HTML processing
            'webdriver-manager>=3.5.0',  # Webdriver management
            'pyppeteer>=1.0.0',  # Headless Chrome/Chromium
            'requests-html>=0.10.0',  # HTML parsing
            'mechanize>=0.4.0',  # Stateful programmatic web browsing
            'scrapy>=2.5.0',  # Web crawling
            'pyquery>=1.4.0',  # jQuery-like HTML parsing
        ],
        'video': [
            'moviepy>=1.0.3',  # Video editing
            'opencv-python>=4.5.0',  # Video processing
            'numpy>=1.21.0',  # Numerical operations
            'ffmpeg-python>=0.2.0',  # Video encoding/decoding
            'pillow>=8.3.0',  # Image processing
            'imageio>=2.9.0',  # Image I/O
            'imageio-ffmpeg>=0.4.5',  # FFmpeg support
            'scikit-image>=0.18.0',  # Image processing
            'scipy>=1.7.0',  # Scientific computing
            'av>=9.0.0',  # PyAV for fast video processing
            'python-vlc>=3.0.0',  # VLC bindings
            'pims>=0.5.0',  # Video frame access
        ],
        'translation': [
            'googletrans>=3.1.0a0',  # Google Translate
            'deep-translator>=1.9.0',  # Multiple translation services
            'langdetect>=1.0.9',  # Language detection
            'polyglot>=16.7.4',  # Natural language processing
            'textblob>=0.17.1',  # Text processing
            'mtranslate>=1.8',  # Microsoft Translator API
            'textblob>=0.17.1',  # Text processing and translation
            'lingua>=4.15.0',  # Language detection
        ],
    },
    description='A comprehensive Python library for AI integration, web development with FastAPI/Flask, advanced encryption, secure database management, file operations, library development, text styling, system management, translation services, video creation, web automation, and cryptocurrency token analysis. Features include JWT handling, password management, rate limiting, input sanitization, and more.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='INEX Team',
    author_email='pbstzidr@ywp.freewebhostmost.com',
    url='https://github.com/AmmarBasha2011/inex_library',
    project_urls={
        'Documentation': 'https://pypi.org/project/inex',
        'Source': 'https://github.com/AmmarBasha2011/inex_library',
        'Tracker': 'https://github.com/AmmarBasha2011/inex_library/issues',
        'Download': 'https://pypi.org/project/inex#files',
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Security :: Cryptography',
        'Topic :: Database',
        'Topic :: System :: Systems Administration',
        'Topic :: Text Processing :: General',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        'Topic :: Text Processing :: General',
        'Topic :: Utilities',
        'Natural Language :: English',
        'Natural Language :: Arabic',
        'Natural Language :: Chinese (Simplified)',
        'Natural Language :: French',
        'Natural Language :: German',
        'Natural Language :: Italian',
        'Natural Language :: Japanese',
        'Natural Language :: Korean',
        'Natural Language :: Portuguese',
        'Natural Language :: Russian',
        'Natural Language :: Spanish',
        'Topic :: Multimedia :: Video',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Framework :: FastAPI',
        'Framework :: Flask',
    ],
    python_requires='>=3.7',
    keywords='ai, gemini, chatgpt, deepseek, web development, fastapi, asgi, server, cors, uvicorn, encryption, aes, blowfish, base64, secure database, sqlite, sqlcipher, flask, crypto, security, blockchain, binance, ethereum, cryptocurrency, file management, file operations, library development, package distribution, text styling, terminal output, jwt, password, rate limiting, input sanitization, 2fa, authentication, wsgi, asgi, gunicorn, nginx, system management, process control, hardware monitoring, translation, language detection, nlp, video creation, video editing, multimedia, web automation, web scraping, browser control',
)



from time import sleep
from sys import stdout

class PrintStyle:
    """This is For Custom Print for Letter"""

    @staticmethod
    def print_one(text: str, second: float = 0.05):
        """This is For Custom Print for Letter

        Args:
            text (str): this is Sentence
            second (float, optional): this is Seconds For Letter. Defaults to 0.05.

        Raises:
            ZeroDivisionError
        """
        
        if len(text) == 0:
            raise ZeroDivisionError
        
        for line in text + '\n':
            stdout.write(line)
            stdout.flush()
            sleep(second)
        
    @staticmethod
    def print_all(text: str, total_time: float = 5):
        """This is For Custom Print for Sentence

        Args:
            text (str): This is Sentence
            total_time (float, optional): This is Seconds For Sentence. Defaults to 5.

        Raises:
            ZeroDivisionError
        """
        
        # حساب الوقت الفاصل بين كل حرف
        if len(text) == 0:
            raise ZeroDivisionError
        else:
            interval = total_time / len(text)
        
        # طباعة النص حرفًا بحرف
        for char in text:
            stdout.write(char)
            stdout.flush()
            sleep(interval)
        
        # طباعة سطر جديد بعد انتهاء النص
        stdout.write('\n')
        stdout.flush()



from googletrans import Translator

class Translate:
    """
        This is for Translate text to any language
    """
    @staticmethod
    def translate_text(text: str, to_lan="en", from_lan="en"):
        """
            Translate text to any language

            Args:
                text (str)
                to_lan (str): To Language. Defaults to "en".
                from_lan (str, optional): From Language. Defaults to "en".

            Returns:
                str: Translated Text
        """
        translator = Translator()
        return translator.translate(text, src=from_lan, dest=to_lan).text


from Crypto.Util.Padding import pad, unpad
from Crypto.Cipher import AES
from Crypto.Cipher import Blowfish
import binascii
import base64

class EnDeCrypt:
    
    @staticmethod
    class aes:
        """
        AES encryption and decryption operations.

        Methods:
        - encrypt: Encrypts a file using AES encryption.
        - decrypt: Decrypts a file encrypted using AES encryption.

        Attributes:
        - None
        """

        def __init__(self):
            pass

        @staticmethod
        def encrypt(file_path="", password=""):
            """
            Encrypts a file using AES encryption.

            Args:
            - file_path: Path to the file to encrypt.
            - password: Password used for encryption.

            Returns:
            - 'done' if encryption is successful.
            - Raises an exception if encryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                key = password.encode('utf-8').ljust(32, b'\0')
                cipher = AES.new(key, AES.MODE_CBC)
                ct_bytes = cipher.encrypt(pad(data, AES.block_size))
                result = cipher.iv + ct_bytes
                output_path = file_path + ".ywpdne"
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                raise e

        @staticmethod
        def decrypt(file_path="", password=""):
            """
            Decrypts a file encrypted using AES encryption.

            Args:
            - file_path: Path to the file to decrypt.
            - password: Password used for decryption.

            Returns:
            - 'done' if decryption is successful.
            - Raises an exception if decryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                key = password.encode('utf-8').ljust(32, b'\0')
                iv = data[:16]
                ct = data[16:]
                cipher = AES.new(key, AES.MODE_CBC, iv)
                result = unpad(cipher.decrypt(ct), AES.block_size)
                output_path = file_path.replace(".ywpdne", "")
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                raise e

    @staticmethod
    class BlowFish:
        """
        Blowfish encryption and decryption operations.

        Methods:
        - encrypt: Encrypts a file using Blowfish encryption.
        - decrypt: Decrypts a file encrypted using Blowfish encryption.

        Attributes:
        - None
        """

        def __init__(self):
            pass

        @staticmethod
        def encrypt(file_path="", password=""):
            """
            Encrypts a file using Blowfish encryption.

            Args:
            - file_path: Path to the file to encrypt.
            - password: Password used for encryption.

            Returns:
            - 'done' if encryption is successful.
            - Returns error message as string if encryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                key = password.encode('utf-8').ljust(32, b'\0')
                cipher = Blowfish.new(key, Blowfish.MODE_CBC)
                ct_bytes = cipher.encrypt(pad(data, Blowfish.block_size))
                result = cipher.iv + ct_bytes
                output_path = file_path + ".ywpdne"
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)

        @staticmethod
        def decrypt(file_path="", password=""):
            """
            Decrypts a file encrypted using Blowfish encryption.

            Args:
            - file_path: Path to the file to decrypt.
            - password: Password used for decryption.

            Returns:
            - 'done' if decryption is successful.
            - Returns error message as string if decryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                key = password.encode('utf-8').ljust(32, b'\0')
                iv = data[:8]
                ct = data[8:]
                cipher = Blowfish.new(key, Blowfish.MODE_CBC, iv)
                result = unpad(cipher.decrypt(ct), Blowfish.block_size)
                output_path = file_path.replace(".ywpdne", "")
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)

    @staticmethod
    class Base64:
        """
        Base64 encoding and decoding operations.

        Methods:
        - encrypt: Encrypts a file using Base64 encoding.
        - decrypt: Decrypts a file encoded using Base64 encoding.

        Attributes:
        - None
        """
        
        def __init__(self):
            pass

        @staticmethod
        def encrypt(file_path=""):
            """
            Encrypts a file using Base64 encoding.

            Args:
            - file_path: Path to the file to encrypt.

            Returns:
            - 'done' if encryption is successful.
            - Returns error message as string if encryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                result = base64.b64encode(data)
                output_path = file_path + ".ywpdne"
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)
            
        @staticmethod
        def decrypt(file_path=""):
            """
            Decrypts a file encoded using Base64 encoding.

            Args:
            - file_path: Path to the file to decrypt.

            Returns:
            - 'done' if decryption is successful.
            - Returns error message as string if decryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                result = base64.b64decode(data)
                output_path = file_path.replace(".ywpdne", "")
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)
            
    @staticmethod
    class Hex:
        """
        Hexadecimal encoding and decoding operations.

        Methods:
        - encrypt: Encrypts a file by converting it to hexadecimal format.
        - decrypt: Decrypts a file from hexadecimal format.

        Attributes:
        - None
        """
        
        def __init__(self):
            pass

        @staticmethod
        def encrypt(file_path=""):
            """
            Encrypts a file by converting it to hexadecimal format.

            Args:
            - file_path: Path to the file to encrypt.

            Returns:
            - 'done' if encryption is successful.
            - Returns error message as string if encryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                result = binascii.hexlify(data)
                output_path = file_path + ".ywpdne"
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)
            
        @staticmethod
        def decrypt(file_path=""):
            """
            Decrypts a file from hexadecimal format.

            Args:
            - file_path: Path to the file to decrypt.

            Returns:
            - 'done' if decryption is successful.
            - Returns error message as string if decryption fails.
            """
            try:
                with open(file_path, 'rb') as f:
                    data = f.read()
                result = binascii.unhexlify(data)
                output_path = file_path.replace(".ywpdne", "")
                with open(output_path, 'wb') as f:
                    f.write(result)
                return 'done'
            except Exception as e:
                return str(e)



from Websites import Websites
from typing import Any

class Crypto:
    @staticmethod
    def token_information(data: Any = "", type: str = 'binance') -> str:
        """
        Opens a web browser with token information based on the type.

        Args:
        - data (Any): Token identifier or data.
        - type (str): Type of token platform ('binance', 'etherum', 'geckoterminal').

        Returns:
        - str: Message indicating if the operation was successful or unsupported type.
        """
        if type == 'binance':
            link = "https://bscscan.com/token/" + str(data)
            Websites.open_website(link)
            return "opened"
        elif type == 'etherum':
            link = "https://etherscan.io/token/" + str(data)
            Websites.open_website(link)
            return "opened"
        elif type == 'geckoterminal':
            link = 'https://ywp.freewebhostmost.com/really/token.php?pool=' + str(data)
            Websites.open_website(link)
            return "opened"
        else:
            return "Unsupported type"